/*
  ******************************************************************************
  * Copyright (c)  2008 - 2022, Shanghai AisinoChip Co.,Ltd .
  * @file    APP.c 
  * @version V1.0.0
  * @date    2022
  * @author  Aisinochip Firmware Team  
  * @brief   UART demo source code.
  ******************************************************************************        
*/
#include "APP.h"
#include "bt_patch_V4931.h"
#include "logger.h"
#include "bsp_uart.h"
#include "bt_phy.h"


//hci_handler_def		BTPHY;

hci_io_stream_def	hci_rece_buf;
uint8_t match = 0;
uint8_t match_addr[6] = {0,0,0,0,0,0};
uint8_t character[2] = {0, 0};
uint8_t ble_send_buf[0xFF];
uint8_t ble_mode	= BLE_SLAVE_MODE;
bool sleep_flag = false;

#define		BTPHY_POWER_LPM		1		// lpm
#define		BTPHY_POWER_DEEPSLEEP	0		// deep sleep

/**
**@name	int8_t rece_msg(hci_handler_def *hci_handler)
**@brief	this function is a user function, which should be completed by
		user. This function should implete to get data from bluetooth module
		and transfer to 'hci_handler->rece_data'.
**@return	0 - received data; others - not received data
**/
int8_t rece_byte(uint8_t *byte)
{
	return (readRingBuff(&UART1_Dat, byte));
}

/**
**@name	int8_t send_msg(hci_handler_def *hci_handler)
**@brief	this function is a user function, which should be completed by
		user. This function should implete to send data to bluetooth module
		from 'hci_handler->send_buf'.
**@return	0 - send success; others - send failed
**/
int8_t send_byte(uint8_t *byte)
{
	bsp_uart_send(config_BTPHY_UART, byte, 1, 0);
//	LOG_BLE("%02x ", *byte);
	return BTPHY_OK;
}

/**
**@name	int8_t send_msg(hci_handler_def *hci_handler)
**@brief	this function is a user function, which should be completed by
		user. This function should implete to return the value of milli-second
**@return	current time of milli-second
**/
uint32_t get_ms(void)
{
	extern volatile uint32_t gu32_SystemCount;
	return gu32_SystemCount;
}

/**
**@name		void event_rece_data_handler(uint8_t *arg)
**@brief	when received a 'HCI_EVENT_SPP_DATA_REP' or 'HCI_EVENT_LE_DATA_REP',
			this function will be called to handle with received data.
**@arg		the pointer to received data, which means as follow
			@ref	hci_received_data_struct_def
**/
void event_rece_data_handler(uint8_t *arg){
	
	uint8_t len;
	
	len = *arg;
	
	LOG_BLE("----------rece---------\r\n");
	for(uint8_t i=0;i<(len-2);i++)
	{
		LOG_BLE("%02x ", arg[i+3]);
		if(i%8 == 7)	LOG_BLE("\r\n");
	}
	LOG_BLE("\r\n-------rece end-------\r\n");
}

/**
**@name		void event_nvram_data_handler(uint8_t *arg)
**@brief	when received a 'HCI_EVENT_NVRAM_REP',this function will be called 
			to handle with received data.
**@arg		the pointer to received data, which means as follow
			@arg[0]: length;	@arg[1]~arg[length]: nvram data
**/
void event_nvram_data_handler(uint8_t *arg){
	__UNUSED(arg);
}

/**
**@name		void event_scan_report_handler(uint8_t *arg)
**@brief	when received a 'HCI_EVENT_SCAN_RES', this function will be called 
			to handle with received data.
**@arg		the pointer to received data, which means as follow
			@ref	hci_advertising_channel_pud_def
**/
void event_scan_report_handler(uint8_t *arg){
	uint8_t match_num = 0;
//	uint8_t aim[] = {"MS66666666"};
	uint8_t aim[] = {"Yichip"};
//	uint8_t aim_addr[6] = {0xf9, 0x4a, 0x69, 0x7b, 0x54, 0x50};
//	uint8_t aim_addr[6] = {0x75, 0xbf, 0x36, 0x2a, 0xd7, 0x5c};
	uint8_t tmp[2] = {1, 0};
	hci_advertising_channel_pud_def	*adv_rep;
	
	adv_rep = (hci_advertising_channel_pud_def *)arg;
/*	
	for(uint8_t i=0;i<6;i++)
	{
		if(aim_addr[i] == adv_rep->address[i])
			match_num++;
		else
			break;
	}
	
	if(match_num >= 6)
	{
		LOG_BLE("1\r\n");
//		btphy_cmd_handler(HCI_CMD_BLE_SCAN, tmp);
//		
//		match = 1;
//		
//		memmove(match_addr, adv_rep->address, 6);
//		return;
	}
	
	match_num = 0;
*/	
	for(uint8_t i=0; i<(adv_rep->length - 6); i++)
	{
		if(aim[match_num] == adv_rep->data[i])
			match_num++;
		else{			
			// when one byte is matched and next one is not matched,
			// this means current advertising is not match.
			if(match_num)
				break;
		}
	}
	
	if(match_num == (sizeof(aim) - 1))		// result is match, string have a null byte ending
	{
		// close scan
		btphy_cmd_handler(HCI_CMD_BLE_SCAN, tmp);
//		UART1_Dat.read = UART1_Dat.write;
//		
//		btphy_cmd_handler(HCI_CMD_BLE_SCAN, tmp);
//		while(btphy_cmd_handler(HCI_CMD_BLE_SCAN, tmp) != BTPHY_OK)
//		{
//			btphy_pollling(&hci_rece_buf);
//		}
		
		match = 1;
		
		memmove(match_addr, adv_rep->address, 6);
		
	}
}

/**
**@name		void event_service_report_handler(uint8_t *arg)
**@brief	when received a 'HCI_EVENT_SERVICE_RES',this function will be called 
			to handle with received data.
**@arg		the pointer to received data, which means as follow
			@arg[0]: length;	@arg[1]~arg[length]: reference as 'hci_service_struct_def'
**/
void event_service_report_handler(uint8_t *arg){
	__UNUSED(arg);
}

/**
**@name		void event_character_report_handler(uint8_t *arg)
**@brief	when received a 'HCI_EVENT_CHARACTER',this function will be called 
			to handle with received data.
**@arg		the pointer to received data, which means as follow
			@arg[0]: length;	@arg[1]~arg[length]: reference as 'hci_character_struct_def'
**/
void event_character_report_handler(uint8_t *arg){
	hci_character_struct_def	*chr_rep;
	uint8_t uuid_len;
	uint32_t id = 0;
	
	chr_rep = (hci_character_struct_def *)(arg+1);
	uuid_len = (*arg) - 5;
	
	if(uuid_len == 4)	//		32bit uuid
	{
		id = (chr_rep->uuid[3] << 24) | (chr_rep->uuid[2] << 16) | (chr_rep->uuid[1] << 8) | (chr_rep->uuid[0]);
	}
	else if(uuid_len == 2)		// 16bit uuid
	{
		id = (chr_rep->uuid[1] << 8) | (chr_rep->uuid[0]);
	}
	else{						// 128bit uuid
		id = (chr_rep->uuid[uuid_len-1] << 24) | (chr_rep->uuid[uuid_len-2] << 16) | 
				(chr_rep->uuid[uuid_len-3] << 8) | (chr_rep->uuid[uuid_len-4]);
	}
	
	if(id == 0x0000FFF2)
	{
		if((chr_rep->attr & 0x08) == 0x08)
		{
			memmove(character, (uint8_t *)(&chr_rep->value_handler), 2);
		
			LOG_BLE("useful character: ");
			LOG_BLE("%02x, %02x\r\n", character[0], character[1]);
		}
	}
}

void loop(void)
{
	btphy_pollling(&hci_rece_buf);
}

void BLE_Init(void)
{
	uint8_t paylaod[256];
	sleep_flag = false;
    
    //rst bt
	HAL_GPIO_WritePin(GPIOC, GPIO_PIN_8, GPIO_PIN_CLEAR);
	System_Delay_MS(20);
	HAL_GPIO_WritePin(GPIOC, GPIO_PIN_8, GPIO_PIN_SET);	    
	System_Delay_MS(100);    
	
	if(btphy_cmd_handler(HCI_CMD_BOOT_RESET, paylaod))
	{
		goto	BLE_Init_ERROR;
	}
	
	if(btphy_cmd_handler(HCI_CMD_BOOT_ECHO, paylaod))
	{
		goto	BLE_Init_ERROR;
	}
	LOG_BLE("BLE Baudrate Confirm successfully\r\n");
	
	if(btphy_cmd_handler(HCI_LOAD_PATCH, (uint8_t *)yc_patch))
	{
		goto	BLE_Init_ERROR;
	}
	
	while(btphy.state.bit.ready == 0){
		btphy_pollling(&hci_rece_buf);
	}
	
	return ;
BLE_Init_ERROR:
	LOG_BLE("BLE Init Failed\r\n");
}

uint8_t BLE_Master_Connection(void)
{
	uint8_t payload[256];
	uint32_t cnt = get_ms() + 60000;
	
	payload[0] = 6;
	memmove(&payload[1], match_addr, 6);
	
	if(btphy_cmd_handler(HCI_CMD_BLE_CREATE_CONN, payload))
	{
		LOG_BLE("connect failed\r\n");
		return 1;
	}
	
	cnt = get_ms() + 60000;
	
	while(btphy.state.bit.ble_con == 0)
	{
		btphy_pollling(&hci_rece_buf);
		
		if(cnt < get_ms())
		{
			LOG_BLE("connect timeout\r\n");
			return 2;
		}
	}
	
	return 0;
}

void BLE_Master_Config(void)
{
	uint8_t payload[256];
	uint32_t cnt = get_ms() + 60000;
	uint32_t timeout = 0;
	
	payload[0] = 0x01;
	payload[1] = 0x01;
	
	if(btphy_cmd_handler(HCI_CMD_BLE_SCAN, payload))
	{
		LOG_BLE("scan device failed\r\n");
		goto	BLE_Master_Config_ERROR;
	}
	
	LOG_BLE("start to scan device\r\n");
	
	while(match == 0)
	{
		btphy_pollling(&hci_rece_buf);
		
		if(cnt < get_ms())
			break;
	}
	
	if(match)
	{
		LOG_BLE("target address: ");
		for(uint8_t i=0;i<6;i++)
			LOG_BLE("%02x ", match_addr[i]);
		LOG_BLE("\r\n");
	}
	else 
	{
		LOG_BLE("no match device\r\n");
		goto	BLE_Master_Config_ERROR;
	}
	
	cnt =  get_ms() + 60000;
	while(cnt > get_ms()){
		if(UART1_Dat.read != UART1_Dat.write)
		{
			btphy_pollling(&hci_rece_buf);
		}
		else{
			if(++timeout > 0xFFFF)
				break;
		}
	}
	
	LOG_BLE("create a connection\r\n");
	
	if(BLE_Master_Connection())
	{
		goto	BLE_Master_Config_ERROR;
	}
	memmove(&ble_send_buf[1], character, 2);
//	ble_send_buf[1] = 0x14;
//	ble_send_buf[2] = 0x00;
	
	LOG_BLE("BLE Connected successfully\r\n");
	
	return ;

BLE_Master_Config_ERROR:
	LOG_BLE("BLE Connct Failed\r\n");
}

void BLE_Slave_Config(void)
{
	uint8_t payload[256];
	uint8_t name[] = {"aisino_ble"};
	uint32_t cnt = get_ms() + 60000;
	
	// set bluetooth name
	*payload = sizeof(name) - 1;
	memmove((payload + 1), name, (*payload));
	
	if(btphy_cmd_handler(HCI_CMD_SET_BLE_NAME, payload))
	{
		LOG_BLE("name config failed\r\n");
		goto	BLE_Slave_Config_ERROR;
	}
	LOG_BLE("name config success\r\n");
	
	System_Delay_MS(100);
	
	*payload = 0x01;
	*(payload + 1) = 0x07;
	// set to be visiable and connectable
	if(btphy_cmd_handler(HCI_CMD_SET_VISIBILITY, payload))
	{
		LOG_BLE("visiable config failed\r\n");
		goto	BLE_Slave_Config_ERROR;
	}
	
	LOG_BLE("visiable config success\r\n");
	
	
	ble_send_buf[1] = 0x0b;
	ble_send_buf[2] = 0x00;
	
	return;
	
BLE_Slave_Config_ERROR:
	LOG_BLE("BLE Slave Config Failed\r\n");
}

void BLE_Sleep_Config(uint8_t mode)
{
	uint8_t tmp[3] = {0};
	
	HAL_GPIO_WritePin(GPIOC, GPIO_PIN_7, GPIO_PIN_SET);		// confirm the mcu won't wake up bt
	
	// LPM mode
	if(mode)
	{
		tmp[0] = 2;
		tmp[1] = 1;
		tmp[2] = 0xB;			// BLE wake up pin GPIO11
		if(btphy_cmd_handler(HCI_CMD_TEST_CLOSE_LPM, tmp))
		{
			LOG_BLE("enter lpm mode failed\r\n");
		}
		
		tmp[0] = 1;
		tmp[1] = 7;
		if(btphy_cmd_handler(HCI_CMD_SET_VISIBILITY, tmp))
		{
			LOG_BLE("enter lpm mode visiable failed\r\n");
		}
	} 
	// Deep Sleep mode
	else{
		tmp[0] = 1;
		tmp[1] = 0xB;		// BLE wake up pin GPIO11
		if(btphy_cmd_handler(HCI_CMD_DEEP_SLEEP, tmp))
		{
			LOG_BLE("Deep sleep failed\r\n");
		}
	}
}


void MCU_Stop_Enter(void)
{
	volatile uint32_t i;
	
	HAL_RTC_PCLK_ENABLE();
	/*ʱлRCH*/
	SCU->CCR1 =SYS_CLK_SRC_RCH;
    /* رXTHXTLRC4M32K ʱ  */
    SCU->RCHCR &= ~(1 << 16);      // RC4M Disable
    SCU->XTHCR &= ~(1 <<  0);     // XTH  Disable
	SCU->PLLCR &= ~(1 << 0);      //PLL Disable

    SCU->STOPCFG   |=  (1 << 0);  // RTCдʹ
    PMU->ANACR &= ~(1 << 0);      // XTL   Disable
    PMU->ANACR &= ~(1 << 8);      // RC32K Disable
	
	/* LVD Disable */
    SCU->LVDCFG &= ~(1 << 0);

    /* STOP ģʽ¹رLDO12RC64M */
    SCU->STOPCFG |= 1 << 9;//STOP Զ LPSTP ģʽ
    SCU->STOPCFG |= 1 << 3;//ֹͣSTOPģʽ£RC64M Զر

    /* All Clock Disable */
    SCU->CLKOCR &= ~(1ul << 31);    // Buzzer2 Disable
    SCU->CLKOCR &= ~(1ul << 23);    // Buzzer1 Disable

    /*RAM Retention ģʽ*/	
	SCU->STOPCFG |=(1<<28);
	
	/*رSRAMδÿռ*/
	//SCU->MEMCFG  |= (14<<16);    //1110 رδõĺ24K
	
    /*Eflash sleepģʽ */
    EFC->CTRL |= 1<<12;
	
    SET_BIT(SCB->SCR, ((uint32_t)SCB_SCR_SLEEPDEEP_Msk));   // Set SLEEPDEEP bit of Cortex System Control Register 
    
    SCU->STOPCFG &= (~BIT11);  // PDDS=0     
    
    SysTick->CTRL = 0;   // disable systick  
	
	__set_PRIMASK(1);    // disable interrupt 
	/*---------------stopģʽ-----------*/
	/* Wait For Interrupt */   
    __WFI();
	
	/*---------------Ѻ-------------*/
	CLEAR_BIT(SCB->SCR, ((uint32_t)SCB_SCR_SLEEPDEEP_Msk)); 
	 	 
	/* ѺEflash˳Sleepģʽ */
	EFC->CTRL &= ~(1<<12);
	for(i = 0; i < 100; i++); //Elash˳Sleepʱ10uS

	__set_PRIMASK(0);    // enable interrupt 

	System_Init();
}

void MCU_Stop_Exit(void)
{
	// wake up the btphy
	HAL_GPIO_WritePin(GPIOC, GPIO_PIN_7, GPIO_PIN_CLEAR);
	System_Delay_MS(1);
	HAL_GPIO_WritePin(GPIOC, GPIO_PIN_7, GPIO_PIN_SET);
}


void MCU_Standby_Enter(void)
{
	HAL_RTC_PCLK_ENABLE();
	/*----------------standyģʽǰ-------------*/
    /* IOλģ */
//    SCU->PABADS= 0xFFFFFFFF;
//    SCU->PCDADS  = 0xFFFFFFFF;

	/*ʱлRC*/
	SCU->CCR1 =SYS_CLK_SRC_RCH;
    /* رXTHXTLRC4M32K ʱ  */
    SCU->RCHCR&= ~(1 << 16);    // RC4M Disable
    SCU->XTHCR &= ~(1 <<  0);    // XTH  Disable
	SCU->PLLCR &= ~(1 << 0);      //PLL Disable

    SCU->STOPCFG   |=  (1 << 0);    // RTCдʹ
    PMU->ANACR &= ~(1 << 0);    // XTL   Disable
    PMU->ANACR &= ~(1 << 8);    // RC32K Disable

    /* RTCCLK Disable */
    PMU->CR1 &= ~(1 << 5);

    /* Standbyģʽ */
//    SCU->STOPCFG |= (1 << 11);//PDDS  DeepSleep ½ STANDBY ģʽϵ
    /* ־Standby */
    PMU->CR1  |= (6 << 9) | (1 << 8) | (1 << 0);
	
    /* PC13 */
    PMU->CR1 |= 1 << 17;//WKUP6~2 ܽŻѹʹ 
    PMU->CR1 |= 1 <<  1;//1ѱ־

    /* Ĭػ */ 
//	PMU->IOCR  &= ~(1 << 6);   //Ϊֽӿ
//    PMU->IOCR  &= ~(1 << 0);
//    PMU->IOCR  |= 1 << 1;
 
    /* LVD Disable */
    SCU->LVDCFG &= ~(1 << 0);

    /* All Clock Disable */
    SCU->CLKOCR &= ~(1ul << 31);    // Buzzer2 Disable
    SCU->CLKOCR &= ~(1ul << 23);    // Buzzer1 Disable
	
	/*-----------standyģʽ,ȴ--------*/
	System_Enter_Standby_Mode();
}


void BLE_Sleep_Test(void)
{
	volatile uint32_t i;
	
//	HAL_RTC_PCLK_ENABLE();
	
	BLE_Sleep_Config(BTPHY_POWER_DEEPSLEEP);				// BTPHY_POWER_DEEPSLEEPBTPHY_POWER_LPM
	
//	System_Delay_MS(5000);
#if 0
	MCU_Stop_Enter();
#else
	MCU_Standby_Enter();
#endif
	
	MCU_Stop_Exit();
	
}

void BLE_Test(void)
{
	
	static uint8_t state = 0;
	static uint32_t cnt;
	static uint8_t index = 0;
	static uint8_t ble_state = 0;
	uint8_t payload[256];
	uint8_t re_connection = 0;

	*payload = 0x01;
	*(payload + 1) = 0x07;
	
	switch(state)
	{
		case 0:
			if(readRingBuff(&UART2_Dat, &ble_send_buf[index+3]) == 0)
			{
				state = 1;
				index++;
				cnt = get_ms() + 5;
			}
			break;
		case 1:
			if(readRingBuff(&UART2_Dat, &ble_send_buf[index+3]) == 0)
			{
				index ++;
				cnt = get_ms() + 5;
			}
			else if(cnt < get_ms())
			{
				state = 2;
			}
			break;
		case 2:
			LOG_BLE("----------send---------\r\n");
			for(uint8_t i=0;i<index;i++)
			{
				LOG_BLE("%02x ", ble_send_buf[i+3]);
				if(i%8 == 7)	LOG_BLE("\r\n");
			}
			LOG_BLE("\r\n-------send end-------\r\n");
			
			ble_send_buf[0] = index+2;
			
			if(btphy.state.bit.ble_con == 0)
			{
				LOG_BLE("BLE has not been connected\r\n");
			}
			else{
				if(btphy_cmd_handler(HCI_CMD_BLE_SEND_DATA, ble_send_buf))
				{
					LOG_BLE("send failed\r\n");
				}
			}
			
			index = 0;
			state = 0;
			break;
		default:
			break;
	}
        
	if(sleep_flag && (HAL_GPIO_ReadPin(GPIOC, GPIO_PIN_13) == GPIO_PIN_SET))
	{
		LOG_SYS("System goto sleep\r\n");
		BLE_Sleep_Test();
		LOG_SYS("System wake up\r\n");
	}
	
	if(ble_state != btphy.state.bit.ble_con)
	{
		ble_state = btphy.state.bit.ble_con;
		
		if(btphy.state.bit.ble_con == 0)		// off line
		{
			if(ble_mode == BLE_SLAVE_MODE)
			{
				if(btphy_cmd_handler(HCI_CMD_SET_VISIBILITY, payload))
				{
					LOG_BLE("visiable config failed\r\n");
				}
				else
					LOG_BLE("visiable config success\r\n");
			}
			else{
				LOG_BLE("ble disconnected\r\n");
			}
		}
	}
    
    if(ble_mode==BLE_MASTER_MODE)
    {
        // re-connection
        if(btphy.state.bit.ble_con == 0)
        {
            while(BLE_Master_Connection())
            {
                System_Delay_MS(100);
                if(++re_connection > 5)
                {
                    LOG_BLE("ble re-connection failed\r\n");
                }
            }
        }
    }       
}



void EXTI_IRQHandler(void)
{
	if(EXTI->PDR & EXTI_LINE_13)
	{
		EXTI->PDR |= EXTI_LINE_13;
                sleep_flag = sleep_flag ? false: true;
		LOG_SYS("EXTI (PC13) is triggered\r\n");
      
	}
}

