#include "bsp_uart.h"
#include "logger.h"

RingBufferTypeDef	UART3_Dat;
RingBufferTypeDef	UART2_Dat;
RingBufferTypeDef	UART1_Dat;
uint8_t uart3_dat_buffer[UART_RECV_DATA_BUFFER_SIZE];
uint8_t uart2_dat_buffer[UART_RECV_DATA_BUFFER_SIZE];
uint8_t uart1_dat_buffer[UART_RECV_DATA_BUFFER_SIZE];

void HAL_UART_MspInit(UART_HandleTypeDef *huart)
{
    /* 
      NOTE: This function should be modified, when the callback is needed,
      the HAL_UART_MspInit can be implemented in the user file.
    */

    /* For Example */
    GPIO_InitTypeDef    GPIO_Uart;
    
	 if (huart->Instance == UART1) 
    {
        /* Enable Clock */
        System_Module_Enable(EN_UART1);  
        System_Module_Enable(EN_GPIOAB);      
        /* Initialization GPIO */
        GPIO_Uart.Pin       = GPIO_PIN_9 | GPIO_PIN_10;
        GPIO_Uart.Mode      = GPIO_MODE_AF_PP;
        GPIO_Uart.Pull      = GPIO_PULLUP;
        GPIO_Uart.Alternate = GPIO_FUNCTION_2;
        
        HAL_GPIO_Init(GPIOA, &GPIO_Uart);   
        
        NVIC_ClearPendingIRQ(UART1_IRQn);
        NVIC_SetPriority(UART1_IRQn, (1U << __NVIC_PRIO_BITS) - 2);  
        NVIC_EnableIRQ(UART1_IRQn);  
		
		RingBufferInit(&UART1_Dat, uart1_dat_buffer, UART_RECV_DATA_BUFFER_SIZE, sizeof(uint8_t));
    }
	
    else if (huart->Instance == UART2) 
    {
        /* Enable Clock */
        System_Module_Enable(EN_UART2);  
        System_Module_Enable(EN_GPIOAB);      
        /* Initialization GPIO */
        GPIO_Uart.Pin       = GPIO_PIN_2 | GPIO_PIN_3;
        GPIO_Uart.Mode      = GPIO_MODE_AF_PP;
        GPIO_Uart.Pull      = GPIO_PULLUP;
        GPIO_Uart.Alternate = GPIO_FUNCTION_2;
        
        HAL_GPIO_Init(GPIOA, &GPIO_Uart);   
        
        NVIC_ClearPendingIRQ(UART2_IRQn);
        NVIC_SetPriority(UART2_IRQn, (1U << __NVIC_PRIO_BITS) - 2);  
        NVIC_EnableIRQ(UART2_IRQn);  
		
		RingBufferInit(&UART2_Dat, uart2_dat_buffer, UART_RECV_DATA_BUFFER_SIZE, sizeof(uint8_t));
    }
	else if(huart->Instance == UART3) 
    {
        /* Enable Clock */
        System_Module_Enable(EN_UART3);  
        System_Module_Enable(EN_GPIOAB);      
        /* Initialization GPIO */
        GPIO_Uart.Pin       = GPIO_PIN_8 | GPIO_PIN_9;
        GPIO_Uart.Mode      = GPIO_MODE_AF_PP;
        GPIO_Uart.Pull      = GPIO_PULLUP;
        GPIO_Uart.Alternate = GPIO_FUNCTION_5;
        
        HAL_GPIO_Init(GPIOB, &GPIO_Uart);   
        
        NVIC_ClearPendingIRQ(UART3_IRQn);
        NVIC_SetPriority(UART3_IRQn, (1U << __NVIC_PRIO_BITS) - 2);  
        NVIC_EnableIRQ(UART3_IRQn);  
		
		RingBufferInit(&UART3_Dat, uart3_dat_buffer, UART_RECV_DATA_BUFFER_SIZE, sizeof(uint8_t));
    }
}


/**
**@name		uint8_t bsp_uart_init(UART_TypeDef *UARTx, uint32_t baud, uint8_t debug)
**@UARTx	indicatie which uart should be init
**@baud 	the baud rate of uart
**@debug	indicate whether support output with printf, 0 - unsupport, 1- support
**@return 	0 success, others failed
**/
uint8_t bsp_uart_init(UART_TypeDef *UARTx, uint32_t baud, uint8_t debug)
{
	UART_HandleTypeDef	UART_Handler;
	
	UART_Handler.Instance			= UARTx;
	
	UART_Handler.Init.BaudRate 		= baud;
	UART_Handler.Init.HwFlowCtl 	= UART_HWCONTROL_NONE;
	UART_Handler.Init.Mode 			= debug ? UART_MODE_TX_RX_DEBUG : UART_MODE_TX_RX;
	UART_Handler.Init.Parity 		= UART_PARITY_NONE;
	UART_Handler.Init.StopBits 		= UART_STOPBITS_1;
	UART_Handler.Init.WordLength 	= UART_WORDLENGTH_8B;
	
	HAL_UART_Init(&UART_Handler);
	HAL_UART_Enable_Disable_FIFO(&UART_Handler, FUNC_DISABLE);

	if(debug)	{
		LOG_UART("%s use to be debug uart\r\n", (UARTx == UART1) ? "UART1" : ((UARTx == UART2) ? "UART2" : "UART3"));
	}
	
	LOG_UART("port init: %s\r\n", (UARTx == UART1) ? "RX: PA10 TX: PA9" : ((UARTx == UART2) ? "RX: PA2 TX: PA3" : "RX: PB9 TX: PB8"));
	
	LOG_UART("%s Init Successfully\r\n\r\n", (UARTx == UART1) ? "UART1" : ((UARTx == UART2) ? "UART2" : "UART3"));
	return 0;
}

/**
**@name		uint32_t bsp_uart_send(uint8_t *buf, uint32_t length, uint32_t timeout)
**@buf		the pointer to data which should be sent
**@length	the length of data which should be sent
**@timeout	indicate the limitation of waiting one byte data sending successfully
**@return	the number of sending data
**/
uint32_t bsp_uart_send(UART_TypeDef *UARTx, uint8_t *buf, uint32_t length, uint32_t timeout)
{
	uint32_t time = timeout;
	uint32_t count = 0;
	
	while(length > count)
	{
		if(UARTx->FR & UART_FR_TXFE)
		{
			UARTx->DR = *(buf++);
			count++;
			time = timeout;
		}
		else{
			if(--time == 0)
				break;
		}
	}
	return count;
}



/*********************************************************************************/
/*********************************************************************************/
/**									@IRQ Handler								**/
/*********************************************************************************/
/*********************************************************************************/

void UART1_IRQHandler(void)
{
	uint8_t dat;
	if(UART1->RIS & UART_RIS_RXI)
	{
		dat = UART1->DR;
		writeRingBuff(&UART1_Dat, &dat);
	}

}

void UART2_IRQHandler(void)
{
	uint8_t dat;
	if(UART2->RIS & UART_RIS_RXI)
	{
		dat = UART2->DR;
		writeRingBuff(&UART2_Dat, &dat);
	}

}

void UART3_IRQHandler(void)
{
	uint8_t dat;
	if(UART3->RIS & UART_RIS_RXI)
	{
		dat = UART3->DR;
		writeRingBuff(&UART3_Dat, &dat);
	}

}


