
#include "bt_phy.h"


btphy_state_def	btphy;



/*************************************************************************************
**************************************************************************************
								Interface function
**************************************************************************************
*************************************************************************************/
/**
**@name	int8_t rece_msg(hci_handler_def *hci_handler)
**@brief	this function is a user function, which should be completed by
		user. This function should implete to get data from bluetooth module
		and transfer to 'hci_handler->rece_data'.
**@return	0 - received data; others - not received data
**/
__weak int8_t rece_byte(uint8_t *byte)
{
	LOG_BTPHY("%s is unexisted, please add it\r\n", __FUNCTION__);
	return BTPHY_OK;
}

/**
**@name	int8_t send_msg(hci_handler_def *hci_handler)
**@brief	this function is a user function, which should be completed by
		user. This function should implete to send data to bluetooth module
		from 'hci_handler->send_buf'.
**@return	0 - send success; others - send failed
**/
__weak int8_t send_byte(uint8_t *byte)
{
	LOG_BTPHY("%s is unexisted, please add it\r\n", __FUNCTION__);
	return BTPHY_OK;
}

/**
**@name	int8_t send_msg(hci_handler_def *hci_handler)
**@brief	this function is a user function, which should be completed by
		user. This function should implete to return the value of milli-second
**@return	current time of milli-second
**/
__weak uint32_t get_ms(void)
{
	LOG_BTPHY("%s is unexisted, please add it\r\n", __FUNCTION__);
	return 0;
}

/**
**@name		void event_rece_data_handler(uint8_t *arg)
**@brief	when received a 'HCI_EVENT_SPP_DATA_REP' or 'HCI_EVENT_LE_DATA_REP',
			this function will be called to handle with received data.
**@arg		the pointer to received data, which means as follow
			@ref	hci_received_data_struct_def
**/
__weak void event_rece_data_handler(uint8_t *arg){
	LOG_BTPHY("%s is unexisted, please add it\r\n", __FUNCTION__);
}

/**
**@name		void event_nvram_data_handler(uint8_t *arg)
**@brief	when received a 'HCI_EVENT_NVRAM_REP',this function will be called 
			to handle with received data.
**@arg		the pointer to received data, which means as follow
			@arg[0]: length;	@arg[1]~arg[length]: nvram data
**/
__weak void event_nvram_data_handler(uint8_t *arg){
	LOG_BTPHY("%s is unexisted, please add it\r\n", __FUNCTION__);
}

/**
**@name		void event_scan_report_handler(uint8_t *arg)
**@brief	when received a 'HCI_EVENT_SCAN_RES', this function will be called 
			to handle with received data.
**@arg		the pointer to received data, which means as follow
			@ref	hci_advertising_channel_pud_def
**/
__weak void event_scan_report_handler(uint8_t *arg){
	LOG_BTPHY("%s is unexisted, please add it\r\n", __FUNCTION__);
}

/**
**@name		void event_service_report_handler(uint8_t *arg)
**@brief	when received a 'HCI_EVENT_SERVICE_RES',this function will be called 
			to handle with received data.
**@arg		the pointer to received data, which means as follow
			@arg[0]: length;	@arg[1]~arg[length]: reference as 'hci_service_struct_def'
**/
__weak void event_service_report_handler(uint8_t *arg){
	LOG_BTPHY("%s is unexisted, please add it\r\n", __FUNCTION__);
}

/**
**@name		void event_character_report_handler(uint8_t *arg)
**@brief	when received a 'HCI_EVENT_CHARACTER',this function will be called 
			to handle with received data.
**@arg		the pointer to received data, which means as follow
			@arg[0]: length;	@arg[1]~arg[length]: reference as 'hci_character_struct_def'
**/
__weak void event_character_report_handler(uint8_t *arg){
	LOG_BTPHY("%s is unexisted, please add it\r\n", __FUNCTION__);
}

/*************************************************************************************
**************************************************************************************
						The part of base function
**************************************************************************************
*************************************************************************************/

/**
**@name		send_msg(uint8_t *dat, uint32_t len)
**@brief	use to send message which will be much more than one byte data
**@dat		the pointer to data which should be sent
**@len		the length of data which should be sent
**/
void send_msg(uint8_t *dat, uint32_t len)
{
	while(len)
	{
		send_byte(dat++);
		len--;
	}
}

/**
**@name		uint8_t wait_for_rece_data(hci_handler_def *hci_handler)
**@brief	This function use to wait rece data. The Waiting time can be configured
		by macro 'HCI_RECE_DATA_TIMEOUT'
**@return	0 - get data successfual; others - rece data timeout
**/
uint8_t wait_for_rece_data(uint8_t *byte)
{
	uint32_t curr_time;
	
	curr_time = get_ms() + HCI_RECE_DATA_TIMEOUT;
	
	while(rece_byte(byte))
	{
		if(curr_time < get_ms())
		{
			return BTPHY_RECE_TIMEOUT;
		}
	}
	
	return BTPHY_OK;
}

/*************************************************************************************
**************************************************************************************
						The part of command send
**************************************************************************************
*************************************************************************************/

/**
**@name		int8_t hci_cmd_ack_check(uint8_t *ack, uint32_t len)
**@brief	use to check cmd respond whether correct
**@ack		the pointer to respond data
**@len		the length of respond data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_ack_check(uint8_t *ack, uint32_t len)
{
	uint8_t tmp[256];
	uint8_t i=0;
//	uint8_t j=0;
	
	for(i=0;i<len;i++)
	{
		if(wait_for_rece_data(&tmp[i]))
		{
			LOG_BTPHY("BTPHY ack rece timeout %d/%d: ", i, len);
			for(i=0;i<len;i++)
			{
				LOG_BTPHY("%02x ", tmp[i]);
			}
			LOG_BTPHY("\r\n");
			return BTPHY_RECE_TIMEOUT;
		}
		/*
		else if(tmp[i] != ack[i]){
			LOG_BTPHY("BTPHY mismatch: ");
			for(j=0;j<i;j++)
			{
				LOG_BTPHY("%02x ", tmp[j]);
			}
			LOG_BTPHY("\r\n");
			
			LOG_BTPHY("target ack: ");
			for(j=0;j<len;j++)
			{
				LOG_BTPHY("%02x ", ack[j]);
			}
			LOG_BTPHY("\r\n");
			return BTPHY_ACK_MIS_MATCH;
		}
		*/
	}
	
	if(memcmp(tmp, ack, len) == 0)
		return BTPHY_OK;
	else
	{
		LOG_BTPHY("BTPHY mismatch: ");
		for(i=0;i<len;i++)
		{
			LOG_BTPHY("%02x ", tmp[i]);
		}
		LOG_BTPHY("\r\n");
		
		LOG_BTPHY("target ack: ");
		for(i=0;i<len;i++)
		{
			LOG_BTPHY("%02x ", ack[i]);
		}
		LOG_BTPHY("\r\n");
		return BTPHY_ACK_MIS_MATCH;
	}
}

/**
**@name		int8_t hci_cmd_reset_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
			in this function, payload is not useful
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_boot_reset_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x00, 0xFC, 0x00};
	uint8_t ack[7] = {0x04, 0x0E, 0x04, 0x01, 0x00, 0xFC, 0x00};
	// send cmd
	send_msg(cmd, 4);
	
	// wait for ack
	return hci_cmd_ack_check(ack, 7);
}

/**
**@name		int8_t hci_cmd_reset_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
			in this function, payload is not useful
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_boot_baud_handler(uint8_t *payload)
{
	uint8_t cmd[6] = {0x01, 0x02, 0xFC, 0x02};
	
	if(*(payload) != 2)
		return BTPHY_CMD_ILLIGLE;
	
	memmove(&cmd[4], (payload+1), 2);
	
	// send cmd
	send_msg(cmd, 6);
	
	// wait for ack
	return BTPHY_OK;
}

/**
**@name		int8_t hci_cmd_yc_echo_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
			in this function, payload is not useful
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_boot_echo_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x05, 0xFC, 0x00};
	uint8_t ack[7] = {0x04, 0x0E, 0x04, 0x01, 0x05, 0xFC, 0x00};
	
	// send cmd
	send_msg(cmd, 4);
	
	// wait for ack
	return hci_cmd_ack_check(ack, 7);
}

/**
**@name		int8_t hci_load_patch_handler(uint8_t *payload)
**@brief
**@paylaod	byte0~byte1: length of data ;	byte2~byten: entity of data
			this payload will be given by Manufacturers
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_load_patch_handler(uint8_t *payload)
{
	uint16_t len_total;
	uint8_t len_cmd;
	uint32_t index = 0;
	uint8_t ret = BTPHY_OK;
	uint8_t ack[7] = {0x04, 0x0E, 0x04, 0x01, 0x00, 0xFC, 0x00};

	len_total = (*(payload+1));
	len_total <<= 8;
	len_total += (*(payload++));
	
	LOG_BTPHY("patch length: %d\r\n", len_total);
	
	len_total -=2;		// get reserved data length
	payload ++;		// get cmd start pointer
	
	while(len_total)
	{
		len_cmd = *(payload++);
		//LOG_BTPHY("cmd %d len: %d\r\n",index++, len_cmd);
		send_msg(payload, len_cmd);
		
		ack[4] = *(payload + 1);
		
		ret = hci_cmd_ack_check(ack, 7);
		
		if(ret)
		{
			LOG_BTPHY("patch download failed: %d\r\n", len_total);
			break;
		}
		
		payload += len_cmd;
		len_total -= (len_cmd + 1);
	}
	LOG_BTPHY("patch download successfully\r\n");

	return ret;
}

/**
**@name		int8_t hci_cmd_set_bt_addr_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
			the address data is little endian mode
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_set_bt_addr_handler(uint8_t *payload)
{
	uint8_t cmd[140] = {0x01, 0x00};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x00, 0x00};
	
	if(*(payload) != 6)
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_set_bt_addr_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
			the address data is little endian mode
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_set_ble_addr_handler(uint8_t *payload)
{
	uint8_t cmd[140] = {0x01, 0x01};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x01, 0x00};
	
	if(*(payload) != 6)
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_ble_set_name_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_ble_set_visiable_handler(uint8_t *payload)
{
	uint8_t cmd[140] = {0x01, 0x02};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x02, 0x00};
	
	if(*(payload) == 0)
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_ble_set_name_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_bt_set_name_handler(uint8_t *payload)
{
	uint8_t cmd[140] = {0x01, 0x03};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x03, 0x00};
	
	if(*(payload) == 0)
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_ble_set_name_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_ble_set_name_handler(uint8_t *payload)
{
	uint8_t cmd[140] = {0x01, 0x04};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x04, 0x00};
	
	if(*(payload) == 0)
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_ble_send_data_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_spp_send_data_handler(uint8_t *payload)
{
	uint8_t cmd[140] = {0x01, 0x05};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x05, 0x00};
	
	if(*(payload) == 0)
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_ble_send_data_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_ble_send_data_handler(uint8_t *payload)
{
	uint8_t cmd[140] = {0x01, 0x09};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x09, 0x00};
	
	if(*(payload) == 0)
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_status_request_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_status_request_handler(uint8_t *payload)
{
	uint8_t cmd[140] = {0x01, 0x0B, 0x00};
	
	send_msg(cmd, 3);
	
	return BTPHY_OK;
	
}

/**
**@name		int8_t hci_cmd_ble_send_data_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_set_pairing_mode_handler(uint8_t *payload)
{
	uint8_t cmd[140] = {0x01, 0x0C};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x0C, 0x00};
	
	if((*(payload) != 1) || (*(payload + 1) > 2))
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_ble_send_data_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_set_pincode_handler(uint8_t *payload)
{
	uint8_t cmd[140] = {0x01, 0x0D};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x0D, 0x00};
	
	if((*(payload) < 1) || (*(payload) > 10))
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_set_uart_flow_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_set_uart_flow_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x0E, 0x01, 0x00};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x0E, 0x00};
	
	if((*(payload) != 1) || (*(payload+1) > 1))
		return BTPHY_CMD_ILLIGLE;
	
	cmd[3] = *(payload + 1);
	
	send_msg(cmd, 4);
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_set_uart_baud_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_set_uart_baud_handler(uint8_t *payload)
{
	uint8_t cmd[10] = {0x01, 0x0F};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x0F, 0x00};
	
	if((*(payload) < 1) || *(payload) > 7)
		return BTPHY_CMD_ILLIGLE;
	
	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 1));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_set_uart_baud_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
			in this function, payload is not useful
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_version_request_handler(uint8_t *payload)
{
	uint8_t cmd[3] = {0x01, 0x10, 0x00};
	
	send_msg(cmd, 3);
	
	return BTPHY_OK;
	
}

/**
**@name		int8_t hci_cmd_set_uart_baud_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
			in this function, payload is not useful
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_bt_disconnect_handler(uint8_t *payload)
{
	uint8_t cmd[3] = {0x01, 0x11, 0x00};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x11, 0x00};
	
	send_msg(cmd, 3);
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_set_uart_baud_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
			in this funciton, payload is not useful
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_ble_disconnect_handler(uint8_t *payload)
{
	uint8_t cmd[3] = {0x01, 0x12, 0x00};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x12, 0x00};
	
	send_msg(cmd, 3);
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_ble_scan_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_ble_scan_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x14, 0x01, 0x00};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x14, 0x00};
	
	if((*(payload) != 1) || (*(payload+1) > 1))
		return BTPHY_CMD_ILLIGLE;
	
	cmd[3] = *(payload + 1);
	
	// send cmd
	send_msg(cmd, 4);
	
	// wait for ack
	return hci_cmd_ack_check(ack, 5);
}

/**
**@name		int8_t hci_cmd_set_nvram_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_set_nvram_handler(uint8_t *payload)
{
	uint8_t cmd[180] = {0x01, 0x26};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x26, 0x00};
	
	if((*payload) != 0xAA)
		return BTPHY_CMD_ILLIGLE;
	
	memmove(&cmd[2], payload, 0xAB);
	
	send_msg(cmd, 173);
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_set_nvram_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_deep_sleep_handler(uint8_t *payload)
{
	uint8_t cmd[5] = {0x01, 0x27, 0x01, 0x8F};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x27, 0x00};
	
	if((*(payload)) != 1)
		return BTPHY_CMD_ILLIGLE;
	
	cmd[3] = *(payload + 1);			// on or off
	
	send_msg(cmd, 4);
	
	return hci_cmd_ack_check(ack, 5);
//	return BTPHY_OK;
	
}

/**
**@name		int8_t hci_cmd_confirm_gkey_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_confirm_gkey_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x28, 0x01};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x28, 0x00};
	
	if((*(payload) != 1) || (*(payload+1) > 1))
		return BTPHY_CMD_ILLIGLE;
	
	cmd[3] = *(payload + 1);
	
	send_msg(cmd, 4);
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_confirm_gkey_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_set_credit_given_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x29, 0x01};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x29, 0x00};
	
	if(*(payload) != 1) 
		return BTPHY_CMD_ILLIGLE;
	
	cmd[3] = *(payload + 1);
	
	send_msg(cmd, 4);
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_confirm_gkey_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_auto_adv_scan_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x2A};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x2A, 0x00};
	
	if((*(payload) < 1) || (*(payload) > 0x3E))
		return BTPHY_CMD_ILLIGLE;
	
	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_power_req_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
			in this function, payload is not useful
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_power_req_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x2B, 0x00};

	send_msg(cmd, 3);
	
	return BTPHY_OK;
	
}

/**
**@name		int8_t hci_cmd_power_set_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_power_set_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x2C, 0x01};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x2C, 0x00};
	
	if((*(payload) != 1) || (*(payload+1) > 1))
		return BTPHY_CMD_ILLIGLE;

	cmd[3] = *(payload + 1);
	
	send_msg(cmd, 4);
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_power_set_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_passkey_entry_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x30, 0x04};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x30, 0x00};
	
	if(*(payload) != 4)
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_power_set_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_le_set_pairing_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x33};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x33, 0x00};
	
	// valid value: 0x00, 0x01, 0x02, 0x81, 0x82, 0x83
	if((*(payload) != 1) || (*(payload + 1) & 0x7C))
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_power_set_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_le_set_adv_data_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x34};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x34, 0x00};
	
	if(*(payload) > 0x20)
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_power_set_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_le_set_scan_data_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x35};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x35, 0x00};
	
	if(*(payload) > 0x20)
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_power_set_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_le_send_conn_updata_req_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x36};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x36, 0x00};
	
	if(*(payload) != 0x08)
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_power_set_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_le_set_adv_parm_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x37};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x37, 0x00};
	
	if(*(payload) > 0x20)
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_power_set_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_le_start_pairing_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x38, 0x00};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x38, 0x00};

	send_msg(cmd, 3);
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_power_set_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_set_tx_power_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x42};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x42, 0x00};
	
	if((*(payload) != 1) || (*(payload + 1) > 4))
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_power_set_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_le_confirm_gkey_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x48};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x48, 0x00};
	
	if(*(payload) != 1)
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_power_set_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_reject_justwork_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x49};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x49, 0x00};
	
	if((*(payload) != 1) || (*(payload + 1) > 1))
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_reset_chip_req_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
			in this function, payload is not useful
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_reset_chip_req_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x51, 0x00};

	send_msg(cmd, 3);
	
	return BTPHY_OK;
	
}

/**
**@name		int8_t hci_cmd_power_set_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_le_set_fixed_passkey_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x61};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x61, 0x00};
	
	if((*(payload) != 5) || (*(payload + 1) > 1))
		return BTPHY_CMD_ILLIGLE;

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_power_set_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_delete_customize_service_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x76, 0x00};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x76, 0x00};

	send_msg(cmd, 3);
	
	return hci_cmd_ack_check(ack, 5);
	
}

/**
**@name		int8_t hci_cmd_power_set_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_add_service_uuid_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x77};

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return BTPHY_OK;
}

/**
**@name		int8_t hci_cmd_power_set_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_add_characteristic_uuid_handler(uint8_t *payload)
{
	uint8_t cmd[4] = {0x01, 0x78};

	memmove(&cmd[2], payload, (*(payload) + 1));
	
	send_msg(cmd, (*(payload) + 3));
	
	return BTPHY_OK;
	
}

/**
**@name		int8_t hci_cmd_reset_chip_req_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
			in this function, payload is not useful
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_le_create_connection_handler(uint8_t *payload)
{
	uint8_t cmd[10] = {0x01, 0x7B, 0x06};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0x7B, 0x00};
	
	if(*(payload) != 0x06)
		return BTPHY_CMD_ILLIGLE;
	
	memmove(&cmd[3], (payload + 1), 6);

	send_msg(cmd, 9);
	
	return hci_cmd_ack_check(ack, 5);;
	
}

/**
**@name		int8_t hci_cmd_power_set_handler(uint8_t *payload)
**@brief
**@paylaod	byte0: length of data ;	byte1~byten: entity of data
**@return 	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_cmd_close_lpm_handler(uint8_t *payload)
{
	uint8_t cmd[5] = {0x01, 0xFF, 0x02, 0x01, 0x8F};
	uint8_t ack[5] = {0x02, 0x06, 0x02, 0xFF, 0x00};

	if((*(payload + 1)) > 1)
		return BTPHY_CMD_ILLIGLE;
	
	cmd[3] = *(payload + 1);
	cmd[4] = *(payload + 2);
	
	send_msg(cmd, 5);
	
	return hci_cmd_ack_check(ack, 5);
	
}
	
/**
**@name		int8_t btphy_event_handler(hci_handler_def *hci_handler)
**@brief	This function use to handler HCI_EVENT
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t btphy_cmd_handler(enum_BTPHY_HCI_CMD_t cmd, uint8_t *payload)
{
	uint8_t ret = BTPHY_OK;
	switch(cmd)
	{
		case HCI_CMD_SET_BT_ADDR:
			ret = hci_cmd_set_bt_addr_handler(payload);
			break;
		case HCI_CMD_SET_BLE_ADDR:
			ret = hci_cmd_set_ble_addr_handler(payload);
			break;
		case HCI_CMD_SET_VISIBILITY:
			ret = hci_cmd_ble_set_visiable_handler(payload);
			break;
		case HCI_CMD_SET_BT_NAME:
			ret = hci_cmd_bt_set_name_handler(payload);
			break;
		case HCI_CMD_SET_BLE_NAME:
			ret = hci_cmd_ble_set_name_handler(payload);
			break;
		case HCI_CMD_SEND_SPP_DATA:
			ret = hci_cmd_spp_send_data_handler(payload);
			break;
		case HCI_CMD_BLE_SEND_DATA:
			ret = hci_cmd_ble_send_data_handler(payload);
			break;
		case HCI_CMD_STATUS_REQUEST:
			ret = hci_cmd_status_request_handler(payload);
			break;
		case HCI_CMD_SET_PAIRING_MODE:
			ret = hci_cmd_set_pairing_mode_handler(payload);
			break;
		case HCI_CMD_SET_PINCODE:
			ret = hci_cmd_set_pincode_handler(payload);
			break;
		case HCI_CMD_SET_UART_FLOW:
			ret = hci_cmd_set_uart_flow_handler(payload);
			break;
		case HCI_CMD_SET_UART_BAUD:
			ret = hci_cmd_set_uart_baud_handler(payload);
			break;
		case HCI_CMD_VERSION_REQUEST:
			ret = hci_cmd_version_request_handler(payload);
			break;
		case HCI_CMD_BT_DISCONNECT:
			ret = hci_cmd_bt_disconnect_handler(payload);
			break;
		case HCI_CMD_BLE_DISCONNECT:
			ret = hci_cmd_ble_disconnect_handler(payload);
			break;
		case HCI_CMD_BLE_SCAN:
			ret = hci_cmd_ble_scan_handler(payload);
			break;
		case HCI_CMD_SET_NVRAM:
			ret = hci_cmd_set_nvram_handler(payload);
			break;
		case HCI_CMD_DEEP_SLEEP:
			ret = hci_cmd_deep_sleep_handler(payload);
			break;
		case HCI_CMD_CONFIRM_GKEY:
			ret = hci_cmd_confirm_gkey_handler(payload);
			break;
		case HCI_CMD_SET_CREDIT_GVIEN:
			ret = hci_cmd_set_credit_given_handler(payload);
			break;
		case HCI_CMD_SET_ADV_DATA:
			ret = hci_cmd_auto_adv_scan_handler(payload);
			break;
		case HCI_CMD_POWER_REQ:
			ret = hci_cmd_power_req_handler(payload);
			break;
		case HCI_CMD_POWER_SET:
			ret = hci_cmd_power_set_handler(payload);
			break;
		case HCI_CMD_PASSKEY_ENTRY:
			ret = hci_cmd_passkey_entry_handler(payload);
			break;
//		case HCI_CMD_SET_GPIO:
//			break;
//		case HCI_CMD_READ_GPIO:
//			break;
		case HCI_CMD_LE_SET_PAIRING:
			ret = hci_cmd_le_set_pairing_handler(payload);
			break;
		case HCI_CMD_LE_SET_ADV_DATA:
			ret = hci_cmd_le_set_adv_data_handler(payload);
			break;
		case HCI_CMD_LE_SET_SCAN_DATA:
			ret = hci_cmd_le_set_scan_data_handler(payload);
			break;
		case HCI_CMD_LE_SEND_CONN_UPDATE_REQ:
			ret = hci_cmd_le_send_conn_updata_req_handler(payload);
			break;
		case HCI_CMD_LE_SET_ADC_PARM:
			ret = hci_cmd_le_set_adv_parm_handler(payload);
			break;
		case HCI_CMD_LE_START_PAIRING:
			ret = hci_cmd_le_start_pairing_handler(payload);
			break;
//		case HCI_CMD_SET_WAKE_GPIO:
//			break;
		case HCI_CMD_SET_TX_POWER:
			ret = hci_cmd_set_tx_power_handler(payload);
			break;
		case HCI_CMD_LE_CONFIRM_KEY:
			ret = hci_cmd_le_confirm_gkey_handler(payload);
			break;
		case HCI_CMD_REJECT_JUSTWORK:
			ret = hci_cmd_reject_justwork_handler(payload);
			break;
		case HCI_CMD_RESET_CHIP_REQ:
			ret = hci_cmd_reset_chip_req_handler(payload);
			break;
		case HCI_CMD_LE_SET_FIXED_PASSKEY:
			ret = hci_cmd_le_set_fixed_passkey_handler(payload);
			break;
		case HCI_CMD_DELETE_CUSTOMIZE_SE:
			ret = hci_cmd_delete_customize_service_handler(payload);
			break;
		case HCI_CMD_ADD_SERVICE_UUID:
			ret = hci_cmd_add_service_uuid_handler(payload);
			break;
		case HCI_CMD_ADD_CHARACTERISTIC:
			ret = hci_cmd_add_characteristic_uuid_handler(payload);
			break;
		case HCI_CMD_BLE_CREATE_CONN:
			ret = hci_cmd_le_create_connection_handler(payload);
			break;
		case HCI_CMD_TEST_CLOSE_LPM:
			ret = hci_cmd_close_lpm_handler(payload);
			break;
		
		case HCI_CMD_BOOT_RESET:
			ret = hci_cmd_boot_reset_handler(payload);
			break;
		case HCI_CMD_BOOT_BAUD:
			ret = hci_cmd_boot_baud_handler(payload);
			break;
		case HCI_CMD_BOOT_ECHO:
			ret = hci_cmd_boot_echo_handler(payload);
			break;

		case HCI_LOAD_PATCH:
			ret = hci_load_patch_handler(payload);
			break;

		default:
			ret = BTPHY_CMD_ILLIGLE;
			break;
	}
	
	return ret;
}


/*************************************************************************************
**************************************************************************************
						The part of event handler
**************************************************************************************
*************************************************************************************/

/**
**@name		int8_t hci_event_scan_res_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with spp connection event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_spp_conn_rep_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	btphy.state.bit.spp_con = 1;
	
	return ret;
}

/**
**@name		int8_t hci_event_le_conn_rep_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with ble connection event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_le_conn_rep_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	btphy.state.bit.ble_con = 1;
	
	return ret;
}

/**
**@name		int8_t hci_event_spp_dis_rep_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with spp disconnected event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_spp_dis_rep_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	btphy.state.bit.spp_con = 0;
	
	return ret;
}

/**
**@name		int8_t hci_event_le_dis_rep_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with ble disconnected event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_le_dis_rep_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	btphy.state.bit.ble_con = 0;
	
	return ret;
}

/**
**@name		int8_t hci_event_cmd_res_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with command respond event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_cmd_res_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	LOG_BTPHY("respond cmd: %02x\r\n", hci_rece->payload[0]);
	
	if(hci_rece->payload[1])		// command impleted failed
	{
		ret = BTPHY_CMD_FAILED;
	}
	
	switch(hci_rece->payload[0])		// check opcode
	{
		case HCI_CMD_VERSION_REQUEST:
			btphy.version = hci_rece->payload[2] + (hci_rece->payload[3] << 8);
			break;
		case HCI_CMD_POWER_REQ:
			break;
		default:
			break;
	}
	
	return ret;
}

/**
**@name		int8_t hci_event_spp_data_rep_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with received data by spp event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_spp_data_rep_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	uint8_t len;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	len = hci_rece->length;
	
	LOG_BTPHY("len %d attribute: %02x %02x\r\n", len,
		hci_rece->payload[0], hci_rece->payload[1]);
	
	for(uint8_t i=2;i<len;i++)
		LOG_BTPHY("%c", hci_rece->payload[i]);
	
	event_rece_data_handler(&hci_rece->length);
	
	return ret;
}

/**
**@name		int8_t hci_event_le_data_rep_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with received data by ble event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_le_data_rep_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	uint8_t len;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	len = hci_rece->length;
	
	LOG_BTPHY("len %d attribute: %02x %02x\r\n", len,
		hci_rece->payload[0], hci_rece->payload[1]);
	
	for(uint8_t i=2;i<len;i++)
		LOG_BTPHY("%c", hci_rece->payload[i]);
	
	event_rece_data_handler(&hci_rece->length);
	
	return ret;
}

/**
**@name		int8_t hci_event_standby_rep_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with standby rep event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_standby_rep_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	btphy.state.bit.ready	= 1;
	
	return ret;
}

/**
**@name		int8_t hci_event_status_rep_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with status rep event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_status_rep_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	// module is visiable or not
	btphy.state.byte |= (hci_rece->payload[0] & 0x1F);
	
	return ret;
}

/**
**@name		int8_t hci_event_nvram_rep_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with nvram rep event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_nvram_rep_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	event_nvram_data_handler(&hci_rece->length);
	
	return ret;
}

/**
**@name		int8_t hci_event_gkey_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with get key event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_gkey_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	return ret;
}

/**
**@name		int8_t hci_event_invalid_packet_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with invalid packet event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_invalid_packet_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	return ret;
}

/**
**@name		int8_t hci_event_get_passkey_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with get passkey event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_get_passkey_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	return ret;
}

/**
**@name		int8_t hci_event_le_tk_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with ble tk event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_le_tk_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	return ret;
}

/**
**@name		int8_t hci_event_le_pairing_state_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with pairing state event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_le_pairing_state_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	return ret;
}

/**
**@name		int8_t hci_event_le_encryption_state_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with encryption state event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_le_encryption_state_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	return ret;
}

/**
**@name		int8_t hci_event_le_gkey_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with get key of BLE event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_le_gkey_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	return ret;
}

/**
**@name		int8_t hci_event_uuid_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with uuid event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_uuid_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	return ret;
}

/**
**@name		int8_t hci_event_scan_res_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with scan report event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_scan_res_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;

	hci_advertising_channel_pud_def	adv_pdu;
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	memmove((uint8_t *)(&adv_pdu), hci_rece->payload, hci_rece->length);
	
	LOG_BTPHY("type: %02x\r\n", adv_pdu.type);
	LOG_BTPHY("length: %02x\r\n", adv_pdu.length);
	LOG_BTPHY("address: ");
	for(uint8_t i=0;i<6;i++)
	{
		LOG_BTPHY("%02x ", adv_pdu.address[i]);
	}
	LOG_BTPHY("\r\n");
	
	LOG_BTPHY("name: ");
	for(uint8_t i=0;i<(adv_pdu.length - 6);i++)
		LOG_BTPHY("%c", adv_pdu.data[i]);
	LOG_BTPHY("\r\n");
	
	event_scan_report_handler((uint8_t *)(&adv_pdu));
	
	return ret;
}

/**
**@name		int8_t hci_event_service_res_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with service report event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_service_res_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	uint8_t len;
	uint8_t index = 0;
	uint8_t num = 1;
	uint8_t arg[0xFF];
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	LOG_BTPHY("%02x ", hci_rece->type);
	LOG_BTPHY("%02x ", hci_rece->opcode);
	LOG_BTPHY("%02x ", hci_rece->length);
	for(uint8_t i=0;i<hci_rece->length;i++)
		LOG_BTPHY("%02x ", hci_rece->payload[i]);
	LOG_BTPHY("\r\n");
	
	len = hci_rece->length;
	
	arg[0] = hci_rece->payload[index++];
	
	LOG_BTPHY("len: %d, %d\r\n", len, arg[0]);
	
	len--;
	
	while(len)
	{
		LOG_BTPHY("service %d: \r\n", num++);
		LOG_BTPHY("start: %02x %02x\r\n", 
			hci_rece->payload[index], hci_rece->payload[index + 1]);
		LOG_BTPHY("end: %02x %02x\r\n", 
			hci_rece->payload[index + 2], hci_rece->payload[index + 3]);
		
		LOG_BTPHY("uuid: ");
		for(uint8_t i = 4; i<(arg[0]);i++)
			LOG_BTPHY("%02x ", hci_rece->payload[index + i]);
		LOG_BTPHY("\r\n");
		
		memmove(&arg[1], &hci_rece->payload[index], arg[0]);
		
		event_service_report_handler(arg);
		
		len -= arg[0];
		index += arg[0];
	}
	
	return ret;
}

/**
**@name		int8_t hci_event_character_handler(hci_io_stream_def *hci_rece)
**@brief	use to handler with characters report event
**@hci_rece	received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t hci_event_character_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	uint8_t len;
	uint8_t index = 0;
	uint8_t num = 1;
	uint8_t arg[0xFF];
	
	LOG_BTPHY("%d, %s\r\n", __LINE__, __FUNCTION__);
	
	LOG_BTPHY("%02x ", hci_rece->type);
	LOG_BTPHY("%02x ", hci_rece->opcode);
	LOG_BTPHY("%02x ", hci_rece->length);
	for(uint8_t i=0;i<hci_rece->length;i++)
		LOG_BTPHY("%02x ", hci_rece->payload[i]);
	LOG_BTPHY("\r\n");
	
	len = hci_rece->length;
	
	arg[0] = hci_rece->payload[index++];
	
	LOG_BTPHY("len: %d, %d\r\n", len, arg[0]);
	
	len--;
	
	while(len)
	{
		LOG_BTPHY("character %d: \r\n", num++);
		LOG_BTPHY("handle: %02x %02x, %02x\r\n", 
			hci_rece->payload[index + 0], hci_rece->payload[index + 1],
			hci_rece->payload[index + 2]);
		LOG_BTPHY("value: %02x %02x\r\n", 
			hci_rece->payload[index + 3], hci_rece->payload[index + 4]);
		
		LOG_BTPHY("uuid: ");
		for(uint8_t i = 5; i<(arg[0]);i++)
			LOG_BTPHY("%02x ", hci_rece->payload[index + i]);
		LOG_BTPHY("\r\n");
		
		memmove(&arg[1], &hci_rece->payload[index], arg[0]);
		
		event_character_report_handler(arg);
		
		len -= arg[0];
		index += arg[0];
	}
	
	return ret;
}

/**
**@name		int8_t btphy_event_handler(hci_handler_def *hci_handler)
**@brief	This function use to handler HCI_EVENT
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t btphy_event_handler(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	switch(hci_rece->opcode)
	{
		case HCI_EVENT_SPP_CONN_REP:
			ret = hci_event_spp_conn_rep_handler(hci_rece);
			break;
		case HCI_EVENT_LE_CONN_REP:
			ret = hci_event_le_conn_rep_handler(hci_rece);
			break;
		case HCI_EVENT_SPP_DIS_REP:
			ret = hci_event_spp_dis_rep_handler(hci_rece);
			break;
		case HCI_EVENT_LE_DIS_REP:
			ret = hci_event_le_dis_rep_handler(hci_rece);
			break;
		case HCI_EVENT_CMD_RES:
			ret = hci_event_cmd_res_handler(hci_rece);
			break;
		case HCI_EVENT_SPP_DATA_REP:
			ret = hci_event_spp_data_rep_handler(hci_rece);
			break;
		case HCI_EVENT_LE_DATA_REP:
			ret = hci_event_le_data_rep_handler(hci_rece);
			break;
		case HCI_EVENT_STANDBY_REQ:
			ret = hci_event_standby_rep_handler(hci_rece);
			break;
		case HCI_EVENT_STATUS_RES:
			ret = hci_event_status_rep_handler(hci_rece);
			break;
		case HCI_EVENT_NVRAM_REP:
			ret = hci_event_nvram_rep_handler(hci_rece);
			break;
		case HCI_EVENT_GKEY:
			ret = hci_event_gkey_handler(hci_rece);
			break;
		case HCI_EVENT_INVALID_PACKET:
			ret = hci_event_invalid_packet_handler(hci_rece);
			break;
		case HCI_EVENT_GET_PASSKEY:
			ret = hci_event_get_passkey_handler(hci_rece);
			break;
		case HCI_EVENT_LE_TK:
			ret = hci_event_le_tk_handler(hci_rece);
			break;
		case HCI_EVENT_LE_PAIRING_STATE:
			ret = hci_event_le_pairing_state_handler(hci_rece);
			break;
		case HCI_EVENT_LE_ENCRYPTION_STATE:
			ret = hci_event_le_encryption_state_handler(hci_rece);
			break;
		case HCI_EVENT_LE_GKEY:
			ret = hci_event_le_gkey_handler(hci_rece);
			break;
		case HCI_EVENT_UUID_HANDLE:
			ret = hci_event_uuid_handler(hci_rece);
			break;
		case HCI_EVENT_SCAN_RES:
			ret = hci_event_scan_res_handler(hci_rece);
			break;
		case HCI_EVENT_SERVICE_RES:
			ret = hci_event_service_res_handler(hci_rece);
			break;
		case HCI_EVENT_CHARACTER:
			ret = hci_event_character_handler(hci_rece);
			break;
		default:
			LOG_BTPHY("%02x ", hci_rece->type);
			LOG_BTPHY("%02x ", hci_rece->opcode);
			LOG_BTPHY("%02x ", hci_rece->length);
			for(uint8_t i=0;i<hci_rece->length - 3;i++)
				LOG_BTPHY("%02x ", hci_rece->payload[i]);
			ret = BTPHY_EVENT_ILLIGLE;
			break;
	}
	return ret;
}

/**
**@name		int8_t btphy_process(hci_handler_def *hci_handler)
**@brief	This function use to handler received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t btphy_rece_process(hci_io_stream_def *hci_rece)
{
	uint8_t ret = BTPHY_OK;
	// other event handler
	switch(hci_rece->type)
	{
		case HCI_EVENT:
			ret = btphy_event_handler(hci_rece);
			break;
		default:
			
			break;
	}
	
	return ret;
}

/**
**@name		int8_t btphy_get_message(hci_handler_def *hci_handler)
**@brief	This function use to get received data
**@return	@ref 'enum_BTPHY_ERROR_CODE_t'
**/
int8_t btphy_get_message(hci_io_stream_def *hci_rece)
{
	uint32_t len = 0;
	
	// wait for get opcode
	if(wait_for_rece_data(&hci_rece->opcode))		
		return BTPHY_RECE_TIMEOUT;
	
	// wait for get len
	if(wait_for_rece_data(&hci_rece->length))		
		return BTPHY_RECE_TIMEOUT;
	
	// wait for get payload
	while(len < hci_rece->length)
	{
		if(wait_for_rece_data(&hci_rece->payload[len++]))		
			return BTPHY_RECE_TIMEOUT;
	}
	
	return btphy_rece_process(hci_rece);
}

/**
**@name		void btphy_pollling(hci_handler_def *hci_handler)
**@brief	This function, which use to handler event about btphy, should 
		be called in main.
**/
void btphy_pollling(hci_io_stream_def *hci_rece)
{	
	uint8_t ret;
	// get rece msg
	if(rece_byte(&hci_rece->type) == 0)
	{
		if(hci_rece->type == HCI_EVENT)
		{
			ret = btphy_get_message(hci_rece);
			if(ret){
				LOG_BTPHY("RECE Dat Error: %d\r\n", ret);
			}
		}
	}
}


/*************************************************************************************
**************************************************************************************
										Init
**************************************************************************************
*************************************************************************************/
void btphy_init(void)
{
	memset((uint8_t *)(&btphy), 0, sizeof(btphy));
}


