
#ifndef		__BT_PHY_H__
#define		__BT_PHY_H__

#include 	"stdint.h"
#include	"stdio.h"
#include 	"string.h"
#include	"logger.h"

#define		HCI_PAYLOAD_SIZE		0xFF

#ifndef		LOG_BTPHY
	#define		LOG_BTPHY(format, ...)    ((void)0)
#endif
	
#ifndef		NULL
	#define		NULL 		0
#endif
	
#define		HCI_RECE_DATA_TIMEOUT	20				// unit: ms
#define		HCI_ACK_WAIT_TIMEOUT	200				// unit: ms


#define		BLE_SLAVE_MODE		1
#define		BLE_MASTER_MODE		0
	
/**
**@def 
**@{
**/
typedef enum
{
	HCI_NULL = 0,
    HCI_CMD = 1,
	HCI_EVENT = 2
}enum_BTPHY_HCI_TYPE_t;

typedef enum
{
	HCI_CMD_SET_BT_ADDR		= 0x00,			// set BT3.0 address	(spp)
	HCI_CMD_SET_BLE_ADDR	= 0x01,			// set BLE Address		(BLE S)
	HCI_CMD_SET_VISIBILITY	= 0x02,			// set visibility and adv	(spp&BLE S)
	HCI_CMD_SET_BT_NAME		= 0x03,			// set name of BT3.0	(spp)
	HCI_CMD_SET_BLE_NAME	= 0x04,			// set name of BLE		(BLE S)
	HCI_CMD_SEND_SPP_DATA	= 0x05,			// send data by BT3.0	(spp)
	HCI_CMD_BLE_SEND_DATA	= 0x09,			// send data by BLE		(BLE M&S)
	HCI_CMD_STATUS_REQUEST	= 0x0B,			// request status of module	(spp&BLE S)
	HCI_CMD_SET_PAIRING_MODE= 0x0C,			// set matching mode	(spp)
	HCI_CMD_SET_PINCODE		= 0x0D,			// set matching code	(spp)
	HCI_CMD_SET_UART_FLOW	= 0x0E,			// set flow control		(spp&BLE M&S)
	HCI_CMD_SET_UART_BAUD	= 0x0F,			// set uart baud of module	(spp&BLE M&S)
	HCI_CMD_VERSION_REQUEST	= 0x10,			// get firmwork version	(spp&BLE M&S)
	HCI_CMD_BT_DISCONNECT	= 0x11,			// BT3.0 disconnect		(spp)
	HCI_CMD_BLE_DISCONNECT	= 0x12,			// BLE disconnect		(BLE M&S)
	HCI_CMD_BLE_SCAN 		= 0x14,			// BLE scan adv			(BLE M)
	HCI_CMD_SET_NVRAM		= 0x26,			// send NV data			(spp&BLE M&S)
	HCI_CMD_DEEP_SLEEP		= 0x27,			// deep sleep mode, in this mode, RAM Data will be lost after wakeup
	HCI_CMD_CONFIRM_GKEY	= 0x28,			// check key under numeric comparison matching mode	(spp&BLE M&S)
	HCI_CMD_SET_CREDIT_GVIEN= 0x29,			// set flow control with spp	(spp)
	HCI_CMD_SET_ADV_DATA	= 0x2A,			// set adv data			(BLE S)
	HCI_CMD_POWER_REQ		= 0x2B,			// query supply source voltage of module	(spp&BLE M&S)
	HCI_CMD_POWER_SET		= 0x2C,			// enabe read supply source voltage of module		(spp&BLE M&S)
	HCI_CMD_PASSKEY_ENTRY	= 0x30,			// input passkey		(BLE S)
	HCI_CMD_SET_GPIO		= 0x31,			// initialize the gpio	(spp&BLE M&S)
	HCI_CMD_READ_GPIO		= 0x32,			// read gpio state		(spp&BLE M&S)
	HCI_CMD_LE_SET_PAIRING	= 0x33,			// set matching mode	(BLE S)
	HCI_CMD_LE_SET_ADV_DATA	= 0x34,			// set adv data			(BLE S)
	HCI_CMD_LE_SET_SCAN_DATA= 0x35,			// set scan data		(BLE S)
	HCI_CMD_LE_SEND_CONN_UPDATE_REQ = 0x36,	// update connecting parameter	(BLE S)
	HCI_CMD_LE_SET_ADC_PARM	= 0x37,			// configure adv parameter	(BLE S)
	HCI_CMD_LE_START_PAIRING= 0x38,			// start to matching	(BLE S)
	HCI_CMD_SET_WAKE_GPIO	= 0x40,			// configure wake up gpio	(BLE S)
	HCI_CMD_SET_TX_POWER	= 0x42,			// configure power of transmit	(BLE S)
	HCI_CMD_LE_CONFIRM_KEY	= 0x48,			// BLE matched by Numeric Comparison
	HCI_CMD_REJECT_JUSTWORK	= 0x49,			// reject that matched by justwork mode
	HCI_CMD_RESET_CHIP_REQ	= 0x51,			// reset chip	(spp&BLE M&S)
	HCI_CMD_LE_SET_FIXED_PASSKEY = 0x61,	// set a fixed passkey	(BLE S)
	HCI_CMD_DELETE_CUSTOMIZE_SE = 0x76,		// delete un-service characters of BLE
	HCI_CMD_ADD_SERVICE_UUID= 0x77,			// add a customize service	(BLE S)
	HCI_CMD_ADD_CHARACTERISTIC = 0x78,		// add a customize characters	(BLE S)
	HCI_CMD_BLE_CREATE_CONN = 0x7B,			// connect with a slaver	(BLE M)
	HCI_CMD_TEST_CLOSE_LPM	= 0xFF,			// close LPM	(BLE S)
	
    HCI_CMD_BOOT_RESET 		= 0xFC00,
	HCI_CMD_BOOT_BAUD 		= 0xFC02,
	HCI_CMD_BOOT_ECHO 		= 0xFC05,
	
	HCI_LOAD_PATCH 			= 0xFFFF,
}enum_BTPHY_HCI_CMD_t;

typedef enum
{
	HCI_EVENT_SPP_CONN_REP	= 0x00,			// BT3.0 connected		(spp)
	HCI_EVENT_LE_CONN_REP	= 0x02,			// BLE connected		(BLE S)
	HCI_EVENT_SPP_DIS_REP	= 0x03,			// BT3.0 disconnected	(spp)
	HCI_EVENT_LE_DIS_REP	= 0x05,			// BLE disconnected		(BLE M&S)
	HCI_EVENT_CMD_RES		= 0x06,			// command has been completed	(spp&BLE M&S)
	HCI_EVENT_SPP_DATA_REP	= 0x07,			// received data from BT3.0	(spp)
	HCI_EVENT_LE_DATA_REP	= 0x08,			// received data from BLE	(BLE M&S)
	HCI_EVENT_STANDBY_REQ	= 0x09,			// module is ready		(BLE M&S)
	HCI_EVENT_STATUS_RES	= 0x0A,			// return module status	(BLE M&S)
	HCI_EVENT_NVRAM_REP		= 0x0D,			// upload NVRAM data 	(BLE M&S)
	HCI_EVENT_GKEY			= 0x0E,			// send key which producted by numeric comparison	(BLE M&S)
	HCI_EVENT_INVALID_PACKET= 0x0F,			// packet format error which sent by MCU	(spp&BLE M&S)
	HCI_EVENT_GET_PASSKEY	= 0x10,			// notify that passkey can be sent by MCU	(BLE S)
	HCI_EVENT_LE_TK			= 0x11,			// notify that passkey should be returned by MCU in BLE PASSKEY match mode	(BLE S)
	HCI_EVENT_LE_PAIRING_STATE = 0x14,		// notify MCU that the matching status of BLE	(BLE S)
	HCI_EVENT_LE_ENCRYPTION_STATE = 0x15,	// notify MCU that current encryption state of BLE	(BLE S)
	HCI_EVENT_LE_GKEY		= 0x1D,			// key producted by numeric comparison	(BLE M&S)
	HCI_EVENT_UUID_HANDLE	= 0x29,			// notify MCU that handle of new uuid	(BLE S)
	HCI_EVENT_SCAN_RES 		= 0x2A,			// report scan event	(BLE M)
	HCI_EVENT_SERVICE_RES 	= 0x50,			// report service		(BLE M)
	HCI_EVENT_CHARACTER 	= 0x51,			// report characters	(BLE M)
	
	
}enum_BTPHY_HCI_EVENT_t;


typedef enum
{
	BTPHY_OK = 0,
	BTPHY_RECE_TIMEOUT,			// 1
	BTPHY_ACK_MIS_MATCH,			// 2
	BTPHY_CMD_ILLIGLE,				// 3
	BTPHY_CMD_FAILED,				// 4
	BTPHY_EVENT_ILLIGLE,			// 5
	BTPHY_STATE_END				// 6
}enum_BTPHY_ERROR_CODE_t;

typedef enum
{
	BTPHY_IDLE = 0,
	BTPHY_WAIT_SEND,
	BTPHY_WAIT_ACK
//	BTPHY_STATE_END
}enum_BTPHY_STATE_t;


typedef enum
{
	ADV_IND = 0,
	ADV_DIRECT_IND,
	ADV_NONCONN_IND,
	SCAN_REQ,
	SCAN_RSP,
	CONNECT_REQ,
	ADC_SCAN_IND
}enum_ADVERTISING_CHANNEL_PDU_TYPE;

/**
**@}
**/

/**
**@def 
**@{
**/
__packed typedef struct{
	uint8_t type;
	uint8_t opcode;
	uint8_t length;
	uint8_t payload[HCI_PAYLOAD_SIZE];
}hci_io_stream_def;

__packed typedef struct hci_advertising_channel_pud_def{
	uint8_t type;
	uint8_t length;
	uint8_t address[6];
	uint8_t data[32];
}hci_advertising_channel_pud_def;

__packed typedef	struct hci_character_struct_def{
	uint16_t chara_handler;
	uint8_t	attr;
	uint16_t value_handler;
	uint8_t uuid[HCI_PAYLOAD_SIZE];
	struct hci_character_struct_def *next;
}hci_character_struct_def;

__packed typedef struct hci_service_struct_def{
	uint16_t	start_handler;
	uint16_t	end_handler;
	uint8_t		uuid[HCI_PAYLOAD_SIZE];
	struct hci_service_struct_def	*next;
}hci_service_struct_def;

__packed typedef struct hci_received_data_struct_def{
	uint8_t length;
	uint8_t attr[2];
	uint8_t data[HCI_PAYLOAD_SIZE];
}hci_received_data_struct_def;

__packed typedef struct{
	uint8_t visiable_30	: 1;
	uint8_t connectable	: 1;
	uint8_t visiable_40	: 1;
	uint8_t spp_con		: 1;
	uint8_t ble_con		: 1;
	uint8_t rsv			: 2;
	uint8_t ready 		: 1;
}bluetooth_state_def;

__packed typedef struct{
	uint16_t version;
	uint8_t mode;
	__packed union{
		bluetooth_state_def	bit;
		uint8_t byte;
	}state;
}btphy_state_def;


/**
**@}
**/

extern btphy_state_def	btphy;

// input and output interface
int8_t rece_byte(uint8_t *byte);
int8_t send_byte(uint8_t *byte);

// time base
uint32_t get_ms(void);

// event handler

// arg[0]: length; arg[1]~arg[2]: attribute;	arg[3]~arg[n]: data
void event_rece_data_handler(uint8_t *arg);

// arg[0]: length; arg[1]~arg[n]: data
void event_nvram_data_handler(uint8_t *arg);

// @ref	'hci_advertising_channel_pud_def'
void event_scan_report_handler(uint8_t *arg);

// arg[0]: length	arg[1]~arg[n]: @ref 'hci_service_struct_def'
void event_service_report_handler(uint8_t *arg);

// arg[0]: length	arg[1]~arg[n]: @ref 'hci_character_struct_def'
void event_character_report_handler(uint8_t *arg);

// should be called by 'main'
void btphy_pollling(hci_io_stream_def *hci_rece);

int8_t btphy_cmd_handler(enum_BTPHY_HCI_CMD_t cmd, uint8_t *payload);


void btphy_init(void);

#endif





